#!/bin/sh

###############################################################################
# $Id: tsgetblocklist.sh 577 2013-01-21 15:05:20Z alein $
#
# Copyright(c) 2006-2011 ThreatSTOP, Inc. All Rights Reserved
#
# NOTICE:  All information contained herein is, and remains the property
# of ThreatSTOP, Inc. and its suppliers, if any. The intellectual and
# technical concepts contained herein are proprietary to ThreatSTOP, Inc.
# and its suppliers and may be covered by U.S. and Foreign Patents, patents
# in process, and are protected by trade secret or copyright law.
# Dissemination of this information or reproduction of this material is
# strictly forbidden unless prior written permission is obtained from
# ThreatSTOP, Inc.
#
#
# Create address-book entries on JunOS of the ThreatSTOP block lists.
# This script is normally run once at setup and every two hours from the cron:
#
###############################################################################

##############################################################
############### GLOBAL DEFENITIONS PART ######################
##############################################################

VERSION="1.00"
MYPID=$$

MAX_BLOCK_LISTS=30
MANDATORY_PARAM_LST="DNSLIST BLOCKLIST ALLOWLIST DEVICE_IP ADDRESS_BOOK_TYPE"
CONFIG="threatstop.conf"
FUNCS="tsfunctions.sh"

# Set verbose level
DEBUG=false
[ ${1-'--'} = DEBUG ] && DEBUG=true

#Changing to working dir
DIR=$(dirname $0)
[ "$DIR" != '.' ] && cd $DIR

# Sourcing the functions file
if [ ! -e $FUNCS ]; then
        echo "ERROR" "Can not load exteranal functions module"
        exit
fi

. ./$FUNCS

##############################################################
################## FUNCTIONS PART ############################
##############################################################


###########################################
#FNdelete
###########################################
FNdelete() {

	local fname=$1

	echoMsg DEBUG "Removing commit script [$fname]"

	echo "configure
                delete system scripts commit file $fname
                commit
        " | /usr/sbin/cli

	return 0
} 

###########################################
#FNcommitXLScode:                         #
###########################################
FNcommitXLScode() {

	local fname=$1

	echoMsg DEBUG "Setting up commit script [$fname]"
	
        echo "configure
                set system scripts commit file $fname
		commit
        " | /usr/sbin/cli

	#echoMsg DEBUG "Deleting commit script [$fname]"
	#rm -f /var/db/scripts/commit/${fname}

	return 0
}

#################################################
#FNloadConfig: Loads treatstop.conf config file #
#################################################
FNloadConfig() {

        echoMsg INFO "Loading mandatory parmaters"

        if [ ! -f $CONFIG ]; then
        	echoMsg ERROR "Cannot find config file: \"$CONFIG\""
                return 1
        fi

	. ./$CONFIG

        #For next check we need to emulate array functionality missing in Bourny shell
        echoMsg INFO "Verifying loaded settings"
        local isOK=1
        for item in $MANDATORY_PARAM_LST; do
                param=$(eval echo \$$item)
                if [ -z "$param" ]; then
                        echoMsg ERROR "Missing mandatory parameter - $item"
                        isOK=0
                fi 
        done

	DNS_PORT=$PORT

        [ $isOK -eq 0 ] && return 1

        return 0
}

############################################
#FNLoadClusterCfg:                         #
############################################
FNLoadClusterCfg() {

	echoMsg INFO "Loading cluster parameters"

	if [ $CLUSTER ]; then
	        SHWNODTP=`echo "show chassis cluster status | display xml" | /usr/sbin/cli`

        	PNODETYPE=`echo "$SHWNODTP" | sed "/banner/!d;s/.* <banner>//;s/<\/banner>//"`
        	NODENUM=`echo "$PNODETYPE" | sed -e "s/[A-Z,a-z]*://;s/{//;s/}//"`
        	NODETYPE=`echo "$PNODETYPE" | sed -e"s/:[A-Z,a-z]*[0-9]//;s/{//;s/}//"`

        	#If node is primary, install threatSTOP rules
        	if [ "$NODETYPE" != "primary" ]; then
                	exit 1
        	fi
	else
		echoMsg INFO "Not cluster mode..."
	fi

	return 0
}

#######################################################################################
# FNLoadDNSsettings : Verifies DNS server connection. Loads new DNS servers list      #
#######################################################################################
FNLoadDNSsettings() {

	#Update dig tool setting 
	if [ ! -z "$NDIG" ]; then
		DIG_TOOL=$(echo $NDIG | awk '{print $1}')
	fi

	echoMsg INFO "Updating DNS setings"
	updateDNSLists 
	[ $? -ne 0 ] && return 1

	#Set DNS server to work with. It will be the first server in the DNSLIST
        #becase after list update this will be the quickest to respond server
        DNS=$(echo $DNSLIST | awk '{print $1}')

	return 0
}

#######################################################################################
#FNLoadDNSsettings: get DNS settings from ThreatSTOP servers                          #
#     Check the availability of threadSTOP DNS servers and select one to work with    #
#     actually we are qoing to use the last one we succeeded to connect during or     #
#     connection test. Also we would like to see if any new DNS servers had been      #
#     detected and add them to config file DNSSERVERS list                            #
#######################################################################################
FNLoadDNSsettingsOLD() {

	local rc
	local rv
	local ndns

	echoMsg INFO "Loading DNS settings"

	for DNS in $DNSLIST; do
	
		#For each defined DNS server do query and see result status 
		echoMsg DEBUG "Running - $NDIG -t apl dns.threatstop.local @''$DNS"
		rc=$($NDIG -p $PORT -t apl dns.threatstop.local @''$DNS 2>&1)
		rv=$?

		#dig utility failure go on and try next server in the list
		if [ $rv -ne 0 ]; then
			echoMsg ERROR "DNS $DNS lookup failed with dig utility fialure"
			continue
                fi

		#dig request return a not NOERROR status go on and try next server in the list
		rc=$(echo $rc | awk '/NOERROR/ { print $0 }')
		if [ -z "$rc" ]; then
			echoMsg ERROR "DNS $DNS lookup returned other than NOERROR status"
                        continue
                fi

		#Everything looks ok for this lookup
		echoMsg DEBUG "DNS $DNS ... [OK]"
                OKDNSLIST="$OKDNSLIST $DNS"
	done

	#Now see if we have at least one DNS server that respond
	if [ $(echo $OKDNSLIST | wc -w) -eq 0 ]; then
                echoMsg ERROR 'No ThreatSTOP DNS servers are reachable'
                return 1        
        fi

	#Now I don't know why but we do prefer to work with the first available DNS sever in the list
	#and not the last inspite of that we already have query result for the last DNS server in 
	#memeory. Do query again and see if we have any new servers that are not in our DNS list
	set $OKDNSLIST
	DNS=$1

	NDNS=$($NDIG -p $PORT -t apl dns.threatstop.local @''$DNS) 
	NDNS=$(echo $NDNS                                    | \
       	       awk '/NOERROR/ { print $0 }'                  | \
               awk 'BEGIN { RS="\;"; FS="\n" } { print $0}'  | \
               grep 'ANSWER SECTION'                         | \
               awk 'BEGIN { RS=" "; FS="\n" } /[0-9]:[0-9]+.[0-9]+.[0-9]+.[0-9]+/ { print $0}' | \
               awk 'BEGIN { FS=":"} {print $2}'              | \
	       sed 's/\/[0-9]*//'
        )

	#If NDNS is empty somethig is wrong abort
	if [ -z "$NDNS" ]; then
		echoMsg ERROR "Failed in DNS request to $DNS server... Can not retrive servers list"
		return 1 
	fi

	#Remove bad address 0.0.0.0
	NDNS=$(echo $NDNS | sed 's/0\.0\.0\.0//')

	for dnadr in $NDNS; do

		echoMsg DEBUG "Verifying $dnadr address"

		#If ND is not in DNSLIST mean this is a new server
	        #add it to the list

		if ! echo $DNSLIST | grep $dnadr 1>/dev/null; then
			echoMsg DEBUG "New server $dnadr detected... Adding to dns list"
			ndns="$ndns $dnadr"		
		fi
	done

	#If we have found new DNS servers update DNS list now
	if [ ! -z "$ndns" ]; then
		DNSLIST="$DNSLIST $ndns"
		cat $CONFIG | sed "s/DNSLIST=.*/DNSLIST=\"$DNSLIST\"/" > /tmp/${CONFIG}.tmp
		mv /tmp/${CONFIG}.tmp $CONFIG
	fi

	return 0
}

############################################
#FNSetBlockPTRrecords:                     #
############################################
FNSetBlockPTRrecords() {

	echoMsg INFO "Setting block PTR records"

	#Clean temporary files
	echoMsg DEBUG "Cleaning up temporary files"
	if [ -f /tmp/tsblock.addr ]; then
        	rm /tmp/tsblock.addr > /dev/null  2>&1
        	rm /tmp/tsadas-body > /dev/null  2>&1
	fi

	#Get PTR address
	PTRBLOCKRECORDS=$($NDIG -p $PORT -t ptr $BLOCKLIST $DIGPR''$DNS | sed -n -e '/IN.*PTR./ s/.*PTR.\(.*\)\./\1/p')
	echoMsg DEBUG "Getting block PTR ADDRESS : $NDIG -p $PORT -t ptr $BLOCKLIST $DIGPR''$DNS"

	#Get ip addresses from APL records by sending request to ThreatSTOP DNS servers
	#Make sure we remove bad ip address 0.0.0.0/0
	for PTR in $PTRBLOCKRECORDS; do
		echoMsg DEBUG "Checking ptr - $NDIG -t apl -p $PORT $PTR $DIGPR''$DNS"
      		$NDIG -t apl -p $PORT $PTR $DIGPR''$DNS | awk '$5~/1:[0-9.]+/ {print $5}'| sed -e "s/1://" -e "s/169\.254\.0\.0\/16//" -e 's/0\.0\.0\.0\/0//' -e '/^$/d' >> /tmp/tsblock.addr

		if [ $? -ne 0 ]; then
			echoMsg ERROR "Failed in dig command... Unable to proceed..."
			return 1
		fi
	done

	return 0
}

############################################################
#FNSetDefaultBlockList: generates blocks to store records  #
#                       solves the 1024 record restriction #
#                       MAX_BLOCK_LISTS=30                 #
############################################################
FNSetDefaultBlockList() {

	echoMsg INFO "Setting default block lists values"

	#Check if ip addresses list has items
	if [ $(cat /tmp/tsblock.addr | wc -w) -eq 0 ]; then
        	echoMsg ERROR 'There is nothing in the block list. Please contact support@threatstop.com for assistance.'
        	echo "0.0.0.1/32" > /tmp/tsblock.addr
		return 1
	fi

	local idx=0
	while [ $idx -le $MAX_BLOCK_LISTS ]; do
		echo "169.254.0.0/16" >> /tmp/tsblock.addr
		idx=$(expr $idx + 1)
	done

	return 0
}

#################################################
#Push IP address into current zones Adress-book #
#################################################
FNCreateBlockSettingZone() {

	local addressName=$1
	local address=$2
	local setNumberB=$3

		#This is old code for root file systems
		echo "  <xsl:call-template name='jcs:emit-change'>
			<xsl:with-param name='content'>
                                 <security>
				 	<address-book>
                                       		<address>
							 <name>ThreatSTOP-block-$addressName</name>
							<ip-prefix>$address</ip-prefix>
                                        	</address>
						<address-set>
                                               		<name>ThreatSTOP-block-$setNumberB</name>
                                               		<address>
                                                       		<name>ThreatSTOP-block-$addressName</name>
                                                	</address>
                                     		</address-set>
					</address-book>
                                 </security>
			</xsl:with-param>
		</xsl:call-template> " >> /tmp/tsadas-body

	return 0
}

####################################################
#Push IP address into current security Adress-book #
####################################################
FNCreateBlockSettingSecurity() {

	local addressName="ThreatSTOP-block-$1"
	local address=$2
        local setAddress="ThreatSTOP-block-$3"

                #This is old code for root file systems
		echo " <xsl:call-template name='jcs:emit-change'>
                       <xsl:with-param name='content'>
				<security>
                                        <address-book>
                                                <name>global</name>
                                                <address>
                                                        <name>$addressName</name>
                                                        <ip-prefix>$address</ip-prefix>
                                                </address>
                                                <address-set>
                                                        <name>$setAddress</name>
                                                        <address>
                                                                <name>$addressName</name>
                                                        </address>
                                                </address-set>
                                        </address-book>
				</security>
                        </xsl:with-param>
                </xsl:call-template>" >> /tmp/tsadas-body
	return 0
}

############################################
#FNProcessBlockList:                       #
############################################
FNProcessBlockList() {

	counter=0
	setNumberB=0
	
	echoMsg INFO "Processing block list ... can take some time ..."
	
	#Process the block list
	#Read the addresses and divide them into blocks of 1000 items each
	#The device has hardware limit of 1024 address-set
	cat /tmp/tsblock.addr | while read address; do
		echoMsg DEBUG "Processing block address $address"
        	let "check=counter % 1000" > /dev/null
        	#let "check=counter % 10" > /dev/null
        	if [ $check -eq 0 ] || [ "$address" =  "169.254.0.0/16" ]; then
                	if [ $setNumberB -eq 30 ]; then
                        	break
                	fi

                	setNumberB=`expr $setNumberB + 1`
        	fi

        	addressName=`echo $address | sed -e 's/[\.\/]/\_/ g'`

		if [ "$ADDRESS_BOOK_TYPE" = "Zones" ]; then
			FNCreateBlockSettingZone "$addressName" "$address" "$setNumberB"
			[ $? -ne 0 ] && return 1
		fi
		
		if [ "$ADDRESS_BOOK_TYPE" = "Security" ]; then
                        FNCreateBlockSettingSecurity $addressName $address $setNumberB 
                        [ $? -ne 0 ] && return 1
                fi

		let "counter=$counter+1" > /dev/null
        	echo $setNumberB > /tmp/ts-setNumberB

	done	

	return 0
}

############################################
#FNCommitBlockListData:                    #
############################################
FNCommitBlockListData() {

	local xslt_script_name=''

	echoMsg INFO "Generating commit BLOCK script"

	#Generating Zones BLOCK commit script
	if [ "$ADDRESS_BOOK_TYPE" = "Zones" ]; then
		xslt_script_name="tsipget-blockZones.xsl"
		if [ -z "$LS_NAME" ]; then
			#Old root system install
			echoMsg DEBUG "Creating tsipget-blockZones.xsl XLST script"
			cat scripts/tsipget-head_block_zones.xsl > /tmp/${xslt_script_name}
		else
			#New logical system install
			echoMsg DEBUG "Creating tsipget-blockZones.xsl XLST script for logical-system $LS_NAME"
			cat scripts/tsipget-head_block_zones.xsl | 
                        sed "s#match=\"security/zones\"#match=\"logical-systems[name=\'$LS_NAME\']/security/zones\"#" \
                        > /tmp/${xslt_script_name}
		fi
	fi

	#Generating Security BLOCK commit script
	if [ "$ADDRESS_BOOK_TYPE" = "Security" ]; then
		xslt_script_name='tsipget-BlockSecurity.xsl'
		if [ -z "$LS_NAME" ]; then
			#Old root system install
			echoMsg DEBUG "Creating tsipget-BlockSecurity.xsl XLST script"
			cat scripts/tsipget-head_block_security.xsl > /tmp/${xslt_script_name}
		else
			#New logical system install
	                echoMsg DEBUG "Creating tsipget-BlockSecurity.xsl XLST script for logical-system $LS_NAME"
			cat scripts/tsipget-head_block_security.xsl | 
                        sed "s#configuration#configuration/logical-systems[name=\'$LS_NAME\']#" > /tmp/${xslt_script_name}
		fi	
	fi

	if [ -z "${xslt_script_name}" ]; then
		echoMsg ERROR "Block ip addresses target list is empty"
		return 1
	fi

	echoMsg DEBUG "Creating XSLT BLOCK list body"
	cat /tmp/tsadas-body >> /tmp/${xslt_script_name}
	if [ $? -ne 0 ]; then
		echoMsg ERROR "Missing BLOCK ip's address list"
		return 1 
	fi
	cat scripts/tsipget_tail.xsl >> /tmp/${xslt_script_name}
	echoMsg DEBUG "Moving /tmp/${xslt_script_name} => /var/db/scripts/commit/${xslt_script_name}"
	cat /tmp/${xslt_script_name} > /var/db/scripts/commit/testBlock.xsl
	mv /tmp/${xslt_script_name} /var/db/scripts/commit/
	if [ $? -ne 0 ]; then
		echoMsg ERROR "Failed to create BLOCK ip's configuration list"
		return 1
	fi

	echoMsg INFO "Commiting created BLOCK configuration"
	BLOCK_FNAME="$xslt_script_name"
	FNcommitXLScode $xslt_script_name 
	[ $? -ne 0 ] && return 1

	FNdelete $xslt_script_name
	[ $? -ne 0 ] && return 1

	return 0
}

############################################
#FNSetAllowPTRrecords:                     #
############################################
FNSetAllowPTRrecords() {

	echoMsg INFO "Setting block PTR records"

	#Clean temporary files
	[ -e /tmp/tsallow.addr ]  && rm /tmp/tsallow.addr &>/dev/null
        [ -e /tmp/tsadasal-body ] && rm /tmp/tsadasal-body &>/dev/null

	#Get PTR list
	echoMsg DEBUG "Getting block PTR ADDRESS : $NDIG p $PORT -t ptr $ALLOWLIST $DIGPR''$DNS"
	PTRALLOWRECORDS=`$NDIG -p $PORT -t ptr $ALLOWLIST $DIGPR''$DNS | sed -n -e        "/$D.*$P./ s/.*$P.\(.*\)\./\1/p"`

	#Get ip address from APL recordsGet ip addresses from APL records by sending request to DNS Threatstop servers
	for PTR in $PTRALLOWRECORDS; do
        	$NDIG -p $PORT -t apl $PTR $DIGPR''$DNS| awk '$5~/1:[0-9.]+/ {print $5}'|sed -e 's/1://'>> /tmp/tsallow.addr
	done

	return 0
}

############################################
#FNSetDefaultAllowList:                    #
############################################
FNSetDefaultAllowList() {

	echoMsg INFO "Setting default Allow lists value"
	
	counterA=0
	setNumberA=0

	#If a user has not set any allow-lists, set the address 64.87.26.147/32

	if [ $(cat /tmp/tsallow.addr | wc -w) -eq 0 ]; then
        	echo "64.87.26.147/32" > /tmp/tsallow.addr
	fi

	return 0
}

#################################################
#Push IP address into current zones Adress-book #
#################################################
FNCreateAllowSettingsZones() {

	local addressNameA=$1
        local address=$2
        local setNumberA=$3

                #This is old code for root file systems
		echo "  <xsl:call-template name='jcs:emit-change'>
        		<xsl:with-param name='content'>
                                <security>
                       			<security-zone>
                       				<name>$UNTRUST</name>
                                        	<address-book>
                                        		<address>
                                               			<name>ThreatSTOP-allow-$addressNameA</name>
                                               			<ip-prefix>$address</ip-prefix>
 	                                              	</address>
        	                                       	<address-set>
                	                                       	<name>ThreatSTOP-allow-$setNumberA</name>
                        	                               	<address>
                                	                       		<name>ThreatSTOP-allow-$addressNameA</name>
                                        	               	</address>
                                                	</address-set>
	                                       </address-book>
        	             		 </security-zone>

 	                                <security-zone>
        	                         	<name>$TRUST</name>
                	                 	<address-book>
                        	         		<address>
                                	 			<name>ThreatSTOP-allow-$addressNameA</name>
                                 				<ip-prefix>$address</ip-prefix>
 	                                		</address>
        	                         		<address-set>
                	                 			<name>ThreatSTOP-allow-$setNumberA</name>
                        	         			<address>
                                	 				<name>ThreatSTOP-allow-$addressNameA</name>
                                 				</address>
        	                      			</address-set>
  	        	                       	</address-book>
                        	  	</security-zone>
				</security>
           		</xsl:with-param>
    		</xsl:call-template> " >> /tmp/tsadasal-body
	return 0
}

####################################################
#Push IP address into current security Adress-book #
####################################################
FNCreateAllowSettingsSecurity() {

        local addressName="ThreatSTOP-allow-$1"
        local address=$2
        local setAddress="ThreatSTOP-allow-$3"

		#This is old code for root systems
		echo " <xsl:call-template name='jcs:emit-change'>
                       <xsl:with-param name='content'>
				<security>
                                        <address-book>
                                                <name>global</name>
                                                <address>
                                                        <name>$addressName</name>
                                                        <ip-prefix>$address</ip-prefix>
                                                </address>
                                                <address-set>
                                                        <name>$setAddress</name>
                                                        <address>
                                                                <name>$addressName</name>
                                                        </address>
                                                </address-set>
                                        </address-book>
				</security>
                        </xsl:with-param>
                </xsl:call-template>" >> /tmp/tsadasal-body
	return 0
}
############################################
FNProcessAllowList() {

	echoMsg INFO "Processing allow lists ...  can take some time ..."

	#We already make sure that allow list is not empty
	#Now process the block list
        #Read the addresses and divide them into blocks of 1000 items each
        #The device has hardware limit of 1024 address-set
	cat /tmp/tsallow.addr | while read address; do

		echoMsg DEBUG "Processing allow address $address"
		let "check=counterA % 1000" > /dev/null

		if [ $check -eq 0 ]; then
                        setNumberA=$(expr $setNumberA + 1)
		fi

		addressName=$(echo $address | sed -e 's/[\.\/]/\_/ g')
		
		if [ "$ADDRESS_BOOK_TYPE" = "Zones" ]; then
			FNCreateAllowSettingsZones $addressNameA $address $setNumberA
			[ $? -ne 0 ] && return 1
		fi

		if [ "$ADDRESS_BOOK_TYPE" = "Security" ]; then
			FNCreateAllowSettingsSecurity $addressName $address $setNumberA 
			[ $? -ne 0 ] && return 1
                fi

		let "counterA=$counterA+1" > /dev/null
                echo $setNumberA > /tmp/ts-setNumberA
	
	done

	return 0
}

############################################
FNCommitAllowListData() {

	local xslt_script_name=''

	echoMsg INFO "Generating commit ALLOW scripts"

	#Generating Zones ALLOW commit script
	if [ "$ADDRESS_BOOK_TYPE" = "Zones" ]; then
                xslt_script_name="tsipget-allowZones.xsl"
		if [ -z "$LS_NAME" ]; then
			#Old root system install
	                echoMsg DEBUG "Creating tsipget-allowZones.xsl XLST script"
        	        cat scripts/tsipget-head_allow_zones.xsl > /tmp/${xslt_script_name}
		else
			#New logical system install
			echoMsg DEBUG "Creating tsipget-allowZones.xsl XLST script for logical-system $LS_NAME"
			cat scripts/tsipget-head_allow_zones.xsl | 
                        sed "s#match=\"security/zones\"#match=\"logical-systems[name=\'$LS_NAME\']/security/zones\"#" \
                        > /tmp/${xslt_script_name}
		fi
        fi

	#Generating Security ALLOW commit script
	if [ "$ADDRESS_BOOK_TYPE" = "Security" ]; then
                xslt_script_name='tsipget-AllowSecurity.xsl'
		if [ -z "$LS_NAME" ]; then
			#Old root system install
	                echoMsg DEBUG "Creating tsipget-AllowSecurity.xsl XLST script"
                	cat scripts/tsipget-head_allow_security.xsl > /tmp/${xslt_script_name}
		else
			#New logical system install
	                echoMsg DEBUG "Creating tsipget-AllowSecurity.xsl XLST script for logical-system $LS_NAME"
			cat scripts/tsipget-head_allow_security.xsl | 
                        sed "s#configuration#configuration/logical-systems[name=\'$LS_NAME\']#" > /tmp/${xslt_script_name}
		fi	
        fi

	if [ -z "${xslt_script_name}" ]; then
		echoMsg ERROR "Allow ip addresses target list is empty"
		return 1
	fi
	
	echoMsg DEBUG "Creating XSLT ALLOW list body"
        cat /tmp/tsadasal-body >> /tmp/${xslt_script_name}
	if [ $? -ne 0 ]; then
                echoMsg ERROR "Missing ALLOW ip's address list"
                return 1
        fi
        cat scripts/tsipget_tail.xsl >> /tmp/${xslt_script_name}
	echoMsg DEBUG "Moving /tmp/${xslt_script_name} => /var/db/scripts/commit/${xslt_script_name}"
	cat /tmp/${xslt_script_name} > /var/db/scripts/commit/testA.xsl
        mv /tmp/${xslt_script_name} /var/db/scripts/commit/
        if [ $? -ne 0 ]; then
                echoMsg ERROR "Failed to create ALLOW ip's configuration list"
                return 1
        fi

        echoMsg INFO "Commiting created ALLOW configuration"
	ALLOW_FNAME="$xslt_script_name"
        FNcommitXLScode $xslt_script_name 
        [ $? -ne 0 ] && return 1

	FNdelete $xslt_script_name
	[ $? -ne 0 ] && return 1

	return 0
}

##############################################################
###################### MAIN PART #############################
##############################################################

echoMsg INFO "Starting getblocklist operation"

FNloadConfig
[ $? -ne 0 ] &&  exit 1

FNLoadClusterCfg
[ $? -ne 0 ] &&  exit 1

FNLoadDNSsettings
[ $? -ne 0 ] &&  exit 1

FNSetBlockPTRrecords
[ $? -ne 0 ] &&  exit 1

FNSetDefaultBlockList
[ $? -ne 0 ] &&  exit 1

FNProcessBlockList
[ $? -ne 0 ] &&  exit 1 

FNCommitBlockListData
[ $? -ne 0 ] &&  exit 1

FNSetAllowPTRrecords
[ $? -ne 0 ] &&  exit 1

FNSetDefaultAllowList
[ $? -ne 0 ] &&  exit 1

FNProcessAllowList
[ $? -ne 0 ] &&  exit 1

FNCommitAllowListData
[ $? -ne 0 ] &&  exit 1

echoMsg INFO "Finished sucsessfully getblocklist operation"
