#!/bin/sh 
###############################################################################
# $Id: tsinstall.sh 579 2013-01-22 13:03:20Z alein $
#
# Copyright(c) 2006-2011 ThreatSTOP, Inc. All Rights Reserved
#
# NOTICE:  All information contained herein is, and remains the property
# of ThreatSTOP, Inc. and its suppliers, if any. The intellectual and
# technical concepts contained herein are proprietary to ThreatSTOP, Inc.
# and its suppliers and may be covered by U.S. and Foreign Patents, patents
# in process, and are protected by trade secret or copyright law.
# Dissemination of this information or reproduction of this material is
# strictly forbidden unless prior written permission is obtained from
# ThreatSTOP, Inc.
#
#
# Install scripts to a JunOS router to enable ThreatSTOP
# Move configuration xsl files into directory commit.
# The script sets the system parameters to send the traffic log every two hours.
# Configures a cronjob to update the block lists every two hours
#
###############################################################################

##############################################################
############### GLOBAL DEFENITIONS PART ######################
##############################################################

VERSION="1.00"
CONFIG="threatstop.conf"
FUNCS="tsfunctions.sh"
GETBLCKLIST_SCRIPT="tsgetblocklist.sh"
OP_PATH='/var/db/scripts/op/'
OP_SCRIPT='scripts/tsinstall.slax'
OP_NAME='tsinstall'
PWD=$(pwd)
CURDATE=$(date +%M)
LOCKFILE="lock.lck"
MINVERS=11;
MYUID=$(id -u)
ADDRESS_BOOK_TYPE=''
ADDRESS_BOOK_TRUST=''
ADDRESS_BOOK_UNTRUST=''
ZONES_LST=''
ZONES_UNTRUST=''
ZONES_TRUST=''
LOGICAL_SYSTEMS_MODE=''
LOGICAL_SYSTEMS_NAME=''

DEBUG="false"
[ "$1" = "DEBUG" ] && DEBUG="true" && shift

#Changing to working dir
DIR=$(dirname $0)
[ "$DIR" != '.' ] && cd $DIR

# Sourcing the functions file
if [ ! -e $FUNCS ]; then
	echo "ERROR" "Can not load exteranal functions module"
	exit
fi
. ./tsfunctions.sh

########################################################################
################## AUXILARY  FUNCTIONS PART ############################
########################################################################

########################################### 
FNcheckScriptsService() {

	local cheks
	local chkact
	local answ

	echoMsg INFO "Verifying script service state"

	cheks=$(echo "configure
                     show system script commit
                    "| /usr/sbin/cli | sed "/inactive:/!d; s/^##//")

        chkact=$(echo "$cheks" | wc -w)
        if [ $chkact -gt 0 ]; then
                echo "$cheks, ThratSTOP commit will not be applied. Continue (y/n): [n]"
                echo -n ""
                read answ
                if [ "$answ" != "y" ]; then
                        echo "Please activate your system script service."
                        return 1
                fi
        fi

	return 0
}

###############################################################
#FNRemoveCommitScript : removes commit script from the system #
#                       to prevent future usage               #
###############################################################
FNRemoveCommitScript() {

        local fname=$1

        echoMsg DEBUG "Removing commit script [$fname] from system configuration"

#        echo "configure
#                delete system scripts commit file $fname
#                commit
#        " | /usr/sbin/cli

        echoMsg DEBUG "Removing [/var/db/scripts/commit/$fname] commit script"
        rm -f /var/db/scripts/commit/${fname}

        return 0
}

##############################################################
#FNcommitTSscript : add supplied commit script to the system #
#                   and do commit over it                    #
############################################################## 
FNcommitTSscript () {
            
	local script_name=$1
	local del_after=$2

        echoMsg INFO "Setting up system installation xsl script"

	echo 
        echo "configure
        	set system scripts commit file $script_name
        	commit check
        	commit
	     " | /usr/sbin/cli
 
	echo

	if [ $del_after = "del" ]; then
                FNRemoveCommitScript $script_name
                [ $? -ne 0 ] && return 1
        fi

        return 0
}

#############################################################
#FNSetOPscripts: set operation list AB query script for     #
#                future usage                               #
#                OP_SCRIPT="scripts/tsinstall.slax"         #
#                OP_PATH="/var/db/scripts/op/"              #
#############################################################
FNSetOPscripts() {

	local op_script=$(basename $OP_SCRIPT)

	echoMsg INFO "Deploying TS auxilary operation script"
	if [ ! -e "$OP_SCRIPT" ]; then
		echoMsg ERROR "Missing mandatory TS auxilary operation script - $OP_SCRIPT"
		return 1
	fi

	echoMsg DEBUG "cp $OP_SCRIPT ${OP_PATH}/${op_script}"
	cp $OP_SCRIPT ${OP_PATH}/${op_script}
	if [ $? -ne 0 ]; then
		echoMsg ERROR "Failed to copy the TS auxilary operation script to ${OP_PATH}/${op_script}"
		return 1
	fi

	echoMsg INFO "Enabling TS auxilary operation script"
	echo "configure
		set system scripts op file $op_script
                commit
        " | /usr/sbin/cli  	

	return 0
}

#############################################################
#FNunSetOPscripts: Run OP AB quiry script to display list   #
#                  of AB used in the system                 #
#                OP_SCRIPT="scripts/tsinstall.slax"         #
#                OP_PATH="/var/db/scripts/op/"              #
#############################################################
FNunSetOPscripts() {

	local op_script=$(basename $OP_SCRIPT)

	echoMsg INFO "Disabeling TS auxilary operation script"
	echo "configure
                delete system scripts op file $op_script
                commit
        " | /usr/sbin/cli

	echoMsg INFO "Removing TS auxilary operation script"
	rm -f ${OP_PATH}/${op_script}
	if [ $? -ne 0 ]; then
		echoMsg WARN "Failed to remove TS auxilary operation script [${OP_PATH}/${op_script}]"
	fi

	return 0
}

##########################################################################################
########################## FLOW FUNCTIONS PART ###########################################
##########################################################################################

###########################################################################
#FNdetectAddressBookOPscript: gives a list of all but global Address Books#
#                             detected in the system.                     #
#                             OP_NAME='tsinstall'                         #
###########################################################################
FNdetectAddressBookOPscript() {

	local res
	local res1
	local res2

	res=$(echo "op $OP_NAME | display xml" | /usr/sbin/cli | sed -e '/[<|{|@]/d' -e '/^$/d' -e 's/^ *//' )

	for line in $res; do

		if echo $line | grep 'LogicalSystemsEnabled' 2>&1 1>/dev/null; then
			LOGICAL_SYSTEMS_MODE=1
			continue
		fi 

		if echo $line | grep 'LogicalSystemsDisabled' 2>&1 1>/dev/null; then
                        LOGICAL_SYSTEMS_MODE=0
                        continue  
                fi

		if echo $line | grep 'lsys:' 2>&1 1>/dev/null; then
                        #Add detected logical system to the list
                        LSYS="$LSYS $line"
                        continue
                fi

		if [ -z "$ADDRESS_BOOK_TYPE" ]; then
			#We need to set it only once. Each output line actually have this value
			ADDRESS_BOOK_TYPE=$(echo $line | awk 'BEGIN {FS=":"} {print $1}')
		fi

		res1=$(echo $line | awk 'BEGIN {FS=":"} {print $2}');
		res2=$(echo $line | awk 'BEGIN {FS=":"} {print $3}');

		if [ "$ADDRESS_BOOK_TYPE" = "Zones" ]; then
			if [ "$res1" = "zone" ]; then
				ZONES_LST="$ZONES_LST $res2"
			fi
		fi

		if [ "$res2" = "trust" ]; then
			[ "$res1" != "zone" ] && ADDRESS_BOOK_TRUST="$ADDRESS_BOOK_TRUST $res1"
			continue
		fi

		if [ "$res2" = "untrust" ]; then
			 [ "$res1" != "zone" ] && ADDRESS_BOOK_UNTRUST="$ADDRESS_BOOK_UNTRUST $res1"
			continue
		fi

		ADDRESS_BOOK_OTHER="$ADDRESS_BOOK_OTHER ${res1}:${res2}"
	done

	return 0
}

##############################################
# Zones Address Book deploy                  #
##############################################
FNSetZonesABtype() {

	local zone
	local opt

	echo "------------------------------------------------------"
	echo "You have the following zones configured on your device"
        echo "------------------------------------------------------"

	echo
	for zone in $ZONES_LST; do
		echo -e "\t$zone"
	done
	echo

	while [ 1 ] ; do	
		echo -n "Please enter the name you use for your 'UNTRUST' zone [untrust] : "
		read opt

		if [ -z "$opt" ]; then
			ZONE_UNTRUST="untrust"
			break
		fi

		if echo $ZONES_LST | grep -q $opt; then
			ZONE_UNTRUST=$opt
			break
		fi
	done

	while [ 1 ]; do
		echo -n "Please enter the name you use for your 'TRUST' zone [trust] : "
                read opt

		if [ -z "$opt" ]; then
                        ZONE_TRUST="trust"
                        break
                fi

                if echo $ZONES_LST | grep -q $opt; then
                        ZONE_TRUST=$opt
                        break;
                fi
        done

	return 0
}

#################################################
# Update xslt commit scripts with user supplied #
# settings for Zones deploy                     #
#################################################
FNSetZonesABtypeSettings() {

	echoMsg INFO "Updating Zones settings "

	#Only update untrust setting if it is non default
	echoMsg INFO "ThreatSTOP block address sets will be created in the [$ZONE_UNTRUST] zone"
        sed "s/untrust/$ZONE_UNTRUST/" scripts/tsinstallZones.xsl            > scripts/nstsinstallZones.xsl 
        sed "s/untrust/$ZONE_UNTRUST/" scripts/tsuninstallZones.xsl          > scripts/nstsuninstallZones.xsl
        sed "s/untrust/$ZONE_UNTRUST/" scripts/tsipget-head_block_zones.xsl > scripts/nstsipget-head_block_zones.xsl
        sed "s/untrust/$ZONE_UNTRUST/" scripts/tsipget-head_allow_zones.xsl   > scripts/nstsipget-head_allow_zones.xsl

	#Only update trust setting if it is non default
        echoMsg INFO "ThreatSTOP address sets will be created in the [$ZONE_TRUST] zone"
	sed "s/trust/$ZONE_TRUST/" scripts/nstsinstallZones.xsl          > scripts/nstsinstallZones-2.xsl
	sed "s/trust/$ZONE_TRUST/" scripts/nstsuninstallZones.xsl        > scripts/nstsuninstallZones-2.xsl
	sed "s/trust/$ZONE_TRUST/" scripts/nstsipget-head_allow_zones.xsl > scripts/nstsipget-head_trustZones-2.xsl

	#Now deploying resulting files
	mv scripts/nstsinstallZones-2.xsl           /var/db/scripts/commit/tsinstallZones.xsl
	mv scripts/nstsuninstallZones-2.xsl         /var/db/scripts/commit/tsuninstallZones.xsl
	mv scripts/nstsipget-head_trustZones-2.xsl  /var/db/scripts/commit/tsipget-head_allow_zones.xsl
	mv scripts/nstsipget-head_block_zones.xsl  /var/db/scripts/commit/tsipget-head_block_zones.xsl

	cat scripts/tsuninstallSecurity.xsl > /var/db/scripts/commit/tsuninstallSecurity.xsl

	#Cleanup leftover temporary files
	rm -f scripts/nstsinstallZones.xsl
	rm -f scripts/nstsuninstallZones.xsl
	rm -f scripts/nstsipget-head_allow_zones.xsl

	return 0
}

#################################################
#FNSetSecurityABtypeSettings:                   #
#################################################
FNSetSecurityABtypeSettings() {

	echoMsg INFO "Updating Security settings"

	cat scripts/tsinstallSecurity.xsl > /var/db/scripts/commit/tsinstallSecurity.xsl
	return 0
}

##########################################################################
#FNrequestUserLSInput : Logical Systems configuration had been detected  #
#                       Ask user to give us LS name to use unless it was #
#                       already supplied on CLI line                     # 
##########################################################################
FNrequestUserLSInput() {

	#Ask user for LS to use
	echo 
	echo "=============================================================="
	echo "WARNING : Logical Systems configurations had been detected !!!"
	echo "=============================================================="

	while [ 1 ]; do

		if [ -z "$LS_NAME" ]; then
        		#We need Logical Sysetm name to continue
                	echo "Please provide logical system name to use"
                        echo -n "Logical System to use [ENTER to use Root System]: "
                	read LS_NAME
        	fi

		if [ -z "$LS_NAME" ]; then
			echo "Using root system to install"
			return 0
		fi

		#Check if supplied name is in detected list. Block execution otherwise
                if ! echo $LSYS | grep $LS_NAME 2>&1 1>/dev/null; then
			echo "ERROR : Supplied Logical System name [$LS_NAME] does not match names configured in the system"
                        echo -n "Retry (Y/n) [Y] : "
                        read opt
        
                        if [ "$opt" = "n" ] || [ "$opr" = "N" ]; then
                                exit
                        fi  

                        echo -n "Logical System to use : "
                        read LS_NAME
                else
                        break
                fi
	done

	return 0
}

##################################################################
#FNrequestUserInput: Interactive user input. Currently used for  #
#                    zones address book file. New format address #
#                    books will be build in global address space #
##################################################################
FNrequestUserInput() {

	local rv
	local opt

	#Dealing with Zones mode address books
	if [ "$1" = "Zones" ]; then

		echo
	        echo "============================================" 
	        echo "Entering address book interactive setup part"
	        echo "============================================"
	        echo

		FNSetZonesABtype	
		rv=$?
		[ $rv -eq 1 ] && return 1
		[ $rv -eq 2 ] && aborting
	fi

	echo -n "Apply settings (Y/N) [Y] : "
	read opt

	if [ "$opt" = N ] || [ "$opt" = n ]; then
		return 2 
	fi
	echo

	return 0
}

###########################################
#FNDetectAddressBooksType:                #
###########################################
FNDetectAddressBooksType() {

	echoMsg INFO "Determining adress book type"
	FNdetectAddressBookOPscript
	[ $? -ne 0 ] && return 1
	
	#If address book is of type - zones use our old deploy flow
	#If address book is of type - security or fresh use global address book 
	#If logical systems were detected - take care
	if [ $LOGICAL_SYSTEMS_MODE -eq 1 ]; then
		FNrequestUserLSInput
		rc=$?
                [ $rc -eq 1 ] && return 1
	fi

	if [ "$ADDRESS_BOOK_TYPE" = "Zones" ]; then
		echoMsg INFO "Zones type address book detected"
		 while [ 1 ]; do
                        FNrequestUserInput Zones
                        rc=$?
                        [ $rc -eq 1 ] && return 1

                        if [ $rc -eq 0 ]; then
				FNSetZonesABtypeSettings
				[ $? -ne 0 ] && return 1
        			
				return 0	
			fi
                done
	fi

	if [ "$ADDRESS_BOOK_TYPE" = "Security" ] || [ "$ADDRESS_BOOK_TYPE" = "Fresh" ]; then
		echoMsg INFO "Security type address book detected"

		#Set Security XSLT files
		FNSetSecurityABtypeSettings
		[ $? -ne 0 ] && return 1
		return 0
	fi
        			
	#If we are here than something is wrong and ADDRESS_BOOK_TYPE detection failed
	echoMsg ERROR "Failed to preoperly detect used address book format"
	return 1;
}

###########################################
#FNUpdateLoggingSettings:                 #
###########################################
FNUpdateLoggingSettings() {
	
	local fname=''
	local fname_tmp=''	

	echoMsg INFO "Updating ThreatSTOP log settings"

	if [ "$ADDRESS_BOOK_TYPE" = "Zones" ]; then
		fname="tsinstallZones.xsl"
		fname_tmp="${fname}.tmp"
	fi

	if [ "$ADDRESS_BOOK_TYPE" = "Security" ]; then
                fname="tsinstallSecurity.xsl"
                fname_tmp="${fname}.tmp"
        fi

	cat /var/db/scripts/commit/$fname | sed "s/ThreatSTOP_log\.log/$DEVICE_IP.log/"  > /var/db/scripts/commit/$fname_tmp
	mv /var/db/scripts/commit/$fname_tmp /var/db/scripts/commit/$fname
	return 0
}

###########################################
#FNVerifySystemTools:                     #
###########################################
FNVerifySystemTools() {

	echoMsg INFO "Verifying required system tools existance"

	#Check if utility either host or dig do exist
	echoMsg INFO "Testing for dig/host utility"
	if [ ! -e /usr/sbin/dig -a ! -e /usr/sbin/host -a ! -e /usr/bin/dig -a ! -e /usr/bin/host ]; then
        	echoMsg ERROR "Neither 'host' nor 'dig' was found in your system. Please verify your version of JUNOS"
       		return 1
	fi

	#Depending on chassis type set utility either host or dig
	echoMsg INFO "Determining appliance software version"
	JUNOSVERS=$(echo 'show version' | cli | sed "/JUNOS/!d;s/[A-Z a-z \[]*//;s/\....\.[0-9]*\]//" | sed '$!N; /^\(.*\)\n\1$/!P; D')
	JUNOSVERSREL=$(echo 'show version' | cli | sed "/JUNOS/!d;s/[A-Z a-z \[]*//;s/..\.[0-9]*\]//;s/[0-9]*\.//" | sed '$!N; /^\(.*\)\n\1$/!P; D')
      
	#If version is less than 11, abort installation. We do not support lower versions anymore becaouse
	#of lists size restriction
	if [ $JUNOSVERS -lt $MINVERS ]; then
		echoMsg ERROR "Unsupported version detected... Should be JUNUOS ${MINVERS}.0 or higher"
		return 1
	fi

	return 0
}

######################################################
#FNConfFileUpdate: update conffile with new settings #
######################################################
FNConfFileUpdate2() {

        echoMsg INFO "Preparing system setings to be uploaded to conf file"
        if [ $JUNOSVERS -lt 10 -a $JUNOSVERSREL -lt 3 ]; then
                NDIG='/usr/bin/host -T -t'
                DIGPR=''
                D=domain
                P=pointer
        else
                if [ ! -e /usr/sbin/dig ]; then
                        echoMsg WARN "No dig utility was found in the system... using host instead"
                        NDIG='/usr/sbin/host -T -t'
                        DIGPR=''
                        D=domain
                        P=pointer
                else
                        NDIG='/usr/sbin/dig +tcp '
                        DIGPR=@
                        D=IN
                        P=PTR
                fi
        fi

        echoMsg INFO  "Updating conf file with system settings"
        echo "# Configuration file for the ThreatSTOP Juniper scripts" > $CONFIG
        echo "" >> $CONFIG

        #Upload back the mandatory parameters we actually read from this file before
        for item in $MANDATORY_PARAM_LST; do
                val=$(eval echo \$$item)
                echo "${item}=\"${val}\"" >> $CONFIG
        done

        echo "NDIG=\"$NDIG\"" >> $CONFIG
        echo "DIGPR=\"$DIGPR\"" >> $CONFIG
        echo "D=\"$D\"" >> $CONFIG
        echo "P=\"$P\"" >> $CONFIG
        echo "ADDRESS_BOOK_TYPE=\"$ADDRESS_BOOK_TYPE\"" >> $CONFIG
        echo "ADDRESS_BOOK_TRUST=\"$ADDRESS_BOOK_TRUST\"" >> $CONFIG
        echo "ADDRESS_BOOK_UNTRUST=\"$ADDRESS_BOOK_UNTRUST\"" >> $CONFIG
	echo "TRUST=\"$ZONE_TRUST\"" >> $CONFIG
	echo "UNTRUST=\"$ZONE_UNTRUST\"" >> $CONFIG
	echo "LS_NAME=\"$LS_NAME\"" >> $CONFIG
	if [ ! -z "$PORT" ]; then 
		echo "PORT=\"$PORT\"" >>$CONFIG
	else
		echo "PORT=\"53\"" >> $CONFIG
	fi

        return 0
}

#######################################################################
#FNcroninst: Setup cron tab entry to run getip's script automatically #
#######################################################################
FNcroninst () {

        echoMsg INFO "Setting crontab entires"

        cronID=$(ps aux | grep cron | awk '{print $2}'| wc -l)
 
        if [ $cronID -gt 1 ]; then

                cronUser=$(/usr/sbin/crontab -l | sed -e "/${GETBLCKLIST_SCRIPT}/d")
		echo "yes" | /usr/sbin/crontab -r
        	echo -e "$cronUser\n$CURDATE */2 * * *      /bin/sh $PWD/${GETBLCKLIST_SCRIPT}" |  /usr/sbin/crontab -
        	echoMsg INFO "Cron job is set to run every 2 hours"

        else
                echoMsg WARN "Cron daemon is not running" 
                return 1
        fi
                          
        return 0
}

###########################################
#FNGetNodeType:                           #
###########################################
FNGetNodeType() {

	echoMsg INFO "Verifying node type"
	SHWNODTP=`echo "
		show chassis cluster status | display xml
	" | cli`

	PNODETYPE=`echo "$SHWNODTP" | sed "/banner/!d;s/.* <banner>//;s/<\/banner>//"`
	NODENUM=`echo "$PNODETYPE" | sed -e "s/[A-Z,a-z]*://;s/{//;s/}//"`
	NODETYPE=`echo "$PNODETYPE" | sed -e"s/:[A-Z,a-z]*[0-9]//;s/{//;s/}//"`

	return 0
}

###########################################
#FNDoNodeSetup:                           #
###########################################
FNDoNodeSetup() {

	echoMsg INFO "Ready to setup Node"

        #Commit the tsinstallXXXXX.xsl script
	if [ "$ADDRESS_BOOK_TYPE" = "Zones" ]; then
	        FNcommitTSscript tsinstallZones.xsl "del"
	fi
          
	if [ "$ADDRESS_BOOK_TYPE" = "Security" ]; then
		FNcommitTSscript tsinstallSecurity.xsl "del"
	fi

	#Turn on debug mode if it is turned on in current script
	[ "$DEBUG" = "true" ] && debug="DEBUG"

        echoMsg INFO "Running get block list script for the first time"
        /bin/sh ./${GETBLCKLIST_SCRIPT} $debug  
        if [ $? -ne 0 ]; then
                echoMsg ERROR "Failed in get block list script"
                return 1
        fi

	echo

        return 0
}

###########################################
FNNodeSetup() {

	echoMsg INFO "Preparing to set configuration on node"

	#If we are running in cluster mode see what node we are curently using
	if [ ! -z "$NODETYPE" ]; then
		echoMsg INFO "Running in cluster mode"
 
		#We are in cluster and current nore is node0
		if [ $NODETYPE = primary -a $NODENUM = node0 ]; then
			echoMsg INFO "Node is set as primaraly and node id is node0... Settings are ok"
			echo "CLUSTER=cluster" >> $PWD/threatstop.conf

			FNDoNodeSetup
                        [ $? -ne 0 ] && return 1

                        return 0
		fi
		
		#We are in cluster and current node is not node0 but curretnly running as primaraly
		#This may be ok if we want to run as stand alone mode but need to verify user intention
                if [ $NODETYPE = "primary" -a $NODENUM != node0 ]; then
 
                        echo -e "\tWARNING !!! Node configuration mismatch detected"
                        echo -e "\tWARNING !!! Current node is running as primaraly"
                        echo -e "\tWARNING !!! but shown as $NODENUM"
                        echo -en "\tInstall as stand alone [Y|n] : "
                        read opt
                        
                        if [ $opt = "n" ]; then
                                echoMsg INFO "Missmatch configuration ... Exiting setup ..."
                                return 1
                        fi

			#User has decided to continue with setup. Basically we need to do stand alone 
			#installation in this case
                        FNDoNodeSetup
                        [ $? -ne 0 ] && return 1
    
                        return 0
                fi

		#If we are in actively in cluster and node and it's state is not primaraly than block this case
		echoMsg ERROR "Current node [$NODENUM] has [$NODETYPE] status"
		echoMsg ERROR "Configuration error... Should be installed on primaraly node"
		return 1
	
        fi

        #If we are here that means we are running in stand alone mode
        echoMsg INFO "Running in standalone mode"

        FNDoNodeSetup
        [ $? -ne 0 ] && return 1

        return 0
}

##############################################################
###################### MAIN PART #############################
##############################################################

clear

echo
echo "****************************************************************"
echo "* Starting ThreatSTOP create address books operation           *"
echo "****************************************************************"
echo

setLock					# Lock instance
[ $? -ne 0 ] && exitting

verifyUserID				# Make sure we have SUID permissions
[ $? -ne 0 ] && exitting

FNloadConfig				# Load if exixsts previous stored configuration otherwise
[ $? -ne 0 ] && exitting		# re-create default settings config file

FNgetCMDparams $*			# Load user supplied CMD parameters. Overload previously 
[ $? -ne 0 ] && exitting		# loaded settings

FNupdateConfFile			# Update configuration file with new settings
[ $? -ne 0 ] && exitting

FNverifySettings			# Verify that we have all mandatory settings
[ $? -ne 0 ] && exitting

FNcheckScriptsService			# See if relevant services are running
[ $? -ne 0 ] && exitting

FNSetOPscripts				# Prepare operation scripts
[ $? -ne 0 ] && exitting

FNDetectAddressBooksType		# Detect and set proper Address Book Usage
[ $? -ne 0 ] && exitting

FNUpdateLoggingSettings 		# Set logging mechanism
[ $? -ne 0 ] && exitting

FNVerifySystemTools			# Detect mandatory system settings
[ $? -ne 0 ] && exitting

FNConfFileUpdate2			# Updating config file with the rest of detected settings 
[ $? -ne 0 ] && exitting

FNGetNodeType				# Detect Node type
[ $? -ne 0 ] && exitting

FNNodeSetup				# Do Node setup
[ $? -ne 0 ] && exitting

FNcroninst				# Update cron setting
[ $? -ne 0 ] && exitting

FNunSetOPscripts			# Remove operation script
[ $? -ne 0 ] && exitting

unLock					# Release lock on instance 
[ $? -ne 0 ] && exitting

echo
echo "****************************************************************"
echo "* ThreatSTOP address books have been created on your device    *"
echo "****************************************************************"
echo

