#!/bin/sh

###############################################################################
# $Id: tspolicycreator.sh 577 2013-01-21 15:05:20Z alein $
#
# Copyright(c) 2006-2011 ThreatSTOP, Inc. All Rights Reserved
#
# NOTICE:  All information contained herein is, and remains the property
# of ThreatSTOP, Inc. and its suppliers, if any. The intellectual and
# technical concepts contained herein are proprietary to ThreatSTOP, Inc.
# and its suppliers and may be covered by U.S. and Foreign Patents, patents
# in process, and are protected by trade secret or copyright law.
# Dissemination of this information or reproduction of this material is
# strictly forbidden unless prior written permission is obtained from
# ThreatSTOP, Inc.
###############################################################################

##############################################################
############### GLOBAL DEFENITIONS PART ######################
##############################################################

VERSION="2.01"
CURDATE=$(date +%M)
LOCKFILE="lock.lck"
LOGFILE="policies.log"
POLICIES_LST="policies.txt"

MYPID=$$

MAX_BLOCK_LISTS=30
MANDATORY_PARAM_LST="DNSLIST BLOCKLIST ALLOWLIST DEVICE_IP ADDRESS_BOOK_TYPE"
CONFIG="threatstop.conf"
FUNCS="tsfunctions.sh"

DEBUG="false"
[ "$1" = "DEBUG" ] && DEBUG="true"

#Changing to working dir
DIR=$(dirname $0)
[ "$DIR" != '.' ] && cd $DIR

# Sourcing the functions file
if [ ! -e $FUNCS ]; then
        echo "ERROR" "Can not load exteranal functions module"
        exit
fi

. ./$FUNCS

##############################################################
################## FUNCTIONS PART ############################
##############################################################

############################################
echoMsg() {

        case $1 in
                ERROR) echo "[ERROR] : $2"
                        ;;
                WARN)  echo "[WARN ] : $2"
                        ;;
                ALERT) echo "[ALERT] : $2"
                        ;;
                INFO)  echo "[INFO ] : $2"
                        ;;
                DEBUG) if [ $DEBUG = true ]; then
                                echo -e "\t[DEBUG] : $2"
                       fi
                        ;;
                *) echo "SCRIPT ERROR : Unrecognized parameter [$1]"
        esac
}

############################################
exiting() {
 
        echo
        echo "****************************************************************"
        echo "*Unrecoverable errors enconted ... aborting...                 *"
        echo "*Please fix then run script again                              *" 
        echo "****************************************************************"
        echo

        exit 1
}

###################################################
cleanup() {

	echo
	echo "****************************************************************"
	echo "* ThreatSTOP policies have not been created on your device     *"
	echo "****************************************************************"
	echo

	exit 0
}

#################################################
#FNloadConfig: Loads treatstop.conf config file #
#################################################
FNloadConfig() {

        echoMsg INFO "Loading mandatory parmaters"

        if [ ! -f $CONFIG ]; then
                echoMsg ERROR "Cannot find config file: \"$CONFIG\""
                return 1
        fi  
 
        . ./$CONFIG   

        #For next check we need to emulate array functionality missing in Bourny shell
        echoMsg INFO "Verifying loaded settings"
        local isOK=1   
        for item in $MANDATORY_PARAM_LST; do
                param=$(eval echo \$$item)
                if [ -z "$param" ]; then
                        echoMsg ERROR "Missing mandatory parameter - $item"
                        isOK=0
                fi
        done

        [ $isOK -eq 0 ] && return 1

	if [ ! -z "$LS_NAME" ]; then
		echoMsg INFO "Running in logical-systems [$LS_NAME] environment"
	fi

        return 0   
}

############################################
FNgetZones() {

	echoMsg INFO "Getting zones settings"
 
	echo
	echo -e "If your UNTRUST zone is not 'untrust', please enter the new name here"
	read -p "[untrust] : " opt
	echo

	UNZONES=$opt
	UNZONES=${UNZONES:-'untrust'}

	echo -e "If your TRUST zone is not 'TRUST', please set it now"
	read -p "[trust] : " opt
	echo

	INZONES=$opt
        INZONES=${INZONES:-'trust'}

	echoMsg INFO "Using zone [$UNZONES] as UNTRUST zone"
	echoMsg INFO "Using zone [$INZONES] as TRUST zone"

	return 0
}

############################################
FNConfigureSets() {

	echoMsg INFO "Getting set configuration"

	ALLOWSETS=`echo 'show configuration' | cli | sed -n 's/.*set \(ThreatSTOP-allow-[0-9]*\).*/\1/p'`
	BLOCKSETS=`echo 'show configuration' | cli | sed -n 's/.*set \(ThreatSTOP-block-[0-9]*\).*/\1/p'`

	echo "configure" > /tmp/blockpol
	echo "configure" >  /tmp/alopol

	return 0
}

############################################
FNSetAllowSet() {

	echoMsg INFO "Creating Allow sets"

	for SET in $ALLOWSETS; do

		echo "set security policies from-zone $INZONE to-zone $UNZONES policy $SET match source-address any" >>/tmp/alopol
		echo "set security policies from-zone $INZONE to-zone $UNZONES policy $SET match destination-address $SET" >>/tmp/alopol
		echo "set security policies from-zone $INZONE to-zone $UNZONES policy $SET match application any" >>/tmp/alopol
		echo "set security policies from-zone $INZONE to-zone $UNZONES policy $SET then permit" >>/tmp/alopol

		echo "set security policies from-zone $UNZONES to-zone $INZONE policy $SET match destination-address any" >>/tmp/alopol
		echo "set security policies from-zone $UNZONES to-zone $INZONE policy $SET match source-address $SET" >>/tmp/alopol
		echo "set security policies from-zone $UNZONES to-zone $INZONE policy $SET match application any" >>/tmp/alopol
		echo "set security policies from-zone $UNZONES to-zone $INZONE policy $SET then permit" >>/tmp/alopol

		if [ "$ISTRUL" ]; then
			echo "insert security policies from-zone $INZONE to-zone $UNZONES policy $SET before policy $ISTRUL" >>/tmp/alopol
		fi

		if [ "$ISTRULTU" ]; then
			echo "insert security policies from-zone $UNZONES to-zone $INZONE policy $SET before policy $ISTRULTU" >>/tmp/alopol
		fi
	done

	return 0
}

############################################
FNSetAllowSetWithLS() {

	echoMsg INFO "Creating Allow sets in [$LS_NAME] logical system"

	for SET in $ALLOWSETS; do

		echo "set logical-systems $LS_NAME security policies from-zone $INZONE to-zone $UNZONES policy $SET match source-address any" >>/tmp/alopol
		echo "set logical-systems $LS_NAME security policies from-zone $INZONE to-zone $UNZONES policy $SET match destination-address $SET" >>/tmp/alopol
		echo "set logical-systems $LS_NAME security policies from-zone $INZONE to-zone $UNZONES policy $SET match application any" >>/tmp/alopol
		echo "set logical-systems $LS_NAME security policies from-zone $INZONE to-zone $UNZONES policy $SET then permit" >>/tmp/alopol

		echo "set logical-systems $LS_NAME security policies from-zone $UNZONES to-zone $INZONE policy $SET match destination-address any" >>/tmp/alopol
		echo "set logical-systems $LS_NAME security policies from-zone $UNZONES to-zone $INZONE policy $SET match source-address $SET" >>/tmp/alopol
		echo "set logical-systems $LS_NAME security policies from-zone $UNZONES to-zone $INZONE policy $SET match application any" >>/tmp/alopol
		echo "set logical-systems $LS_NAME security policies from-zone $UNZONES to-zone $INZONE policy $SET then permit" >>/tmp/alopol

		if [ "$ISTRUL" ]; then
			echo "insert logical-systems $LS_NAME security policies from-zone $INZONE to-zone $UNZONES policy $SET before policy $ISTRUL" >>/tmp/alopol
		fi

		if [ "$ISTRULTU" ]; then
			echo "insert logical-systems $LS_NAME security policies from-zone $UNZONES to-zone $INZONE policy $SET before policy $ISTRULTU" >>/tmp/alopol
		fi
	done

	return 0
}

############################################
FNSetBlockSet() {

	echoMsg INFO "Creating Block sets"

	for SET in $BLOCKSETS; do

		echo "set security policies from-zone $INZONE to-zone $UNZONES policy $SET match source-address any" >>/tmp/blockpol
		echo "set security policies from-zone $INZONE to-zone $UNZONES policy $SET match destination-address $SET" >>/tmp/blockpol
		echo "set security policies from-zone $INZONE to-zone $UNZONES policy $SET match application any" >>/tmp/blockpol
		echo "set security policies from-zone $INZONE to-zone $UNZONES policy $SET then deny" >>/tmp/blockpol
		echo "set security policies from-zone $INZONE to-zone $UNZONES policy $SET then log session-init" >>/tmp/blockpol
		echo "set security policies from-zone $INZONE to-zone $UNZONES policy $SET then count" >>/tmp/blockpol

		echo "set security policies from-zone $UNZONES to-zone $INZONE policy $SET match destination-address any" >>/tmp/blockpol
		echo "set security policies from-zone $UNZONES to-zone $INZONE policy $SET match source-address $SET" >>/tmp/blockpol
		echo "set security policies from-zone $UNZONES to-zone $INZONE policy $SET match application any" >>/tmp/blockpol
		echo "set security policies from-zone $UNZONES to-zone $INZONE policy $SET then deny" >>/tmp/blockpol
		echo "set security policies from-zone $UNZONES to-zone $INZONE policy $SET then log session-init" >>/tmp/blockpol
		echo "set security policies from-zone $UNZONES to-zone $INZONE policy $SET then count" >>/tmp/blockpol

		if [ "$ISTRUL" ]; then
			echo "insert security policies from-zone $INZONE to-zone $UNZONES policy $SET before policy $ISTRUL" >>/tmp/blockpol
		fi

		if [ "$ISTRULTU" ]; then
			echo "insert security policies from-zone $UNZONES to-zone $INZONE policy $SET before policy $ISTRULTU" >>/tmp/blockpol
		fi
	done

	return 0
}

############################################
FNSetBlockSetWithLS() {

	echoMsg INFO "Creating Block sets in [$LS_NAME] logical system"

	for SET in $BLOCKSETS; do

		echo "set logical-systems $LS_NAME security policies from-zone $INZONE to-zone $UNZONES policy $SET match source-address any" >>/tmp/blockpol
		echo "set logical-systems $LS_NAME security policies from-zone $INZONE to-zone $UNZONES policy $SET match destination-address $SET" >>/tmp/blockpol
		echo "set logical-systems $LS_NAME security policies from-zone $INZONE to-zone $UNZONES policy $SET match application any" >>/tmp/blockpol
		echo "set logical-systems $LS_NAME security policies from-zone $INZONE to-zone $UNZONES policy $SET then deny" >>/tmp/blockpol
		echo "set logical-systems $LS_NAME security policies from-zone $INZONE to-zone $UNZONES policy $SET then log session-init" >>/tmp/blockpol
		echo "set logical-systems $LS_NAME security policies from-zone $INZONE to-zone $UNZONES policy $SET then count" >>/tmp/blockpol

		echo "set logical-systems $LS_NAME security policies from-zone $UNZONES to-zone $INZONE policy $SET match destination-address any" >>/tmp/blockpol
		echo "set logical-systems $LS_NAME security policies from-zone $UNZONES to-zone $INZONE policy $SET match source-address $SET" >>/tmp/blockpol
		echo "set logical-systems $LS_NAME security policies from-zone $UNZONES to-zone $INZONE policy $SET match application any" >>/tmp/blockpol
		echo "set logical-systems $LS_NAME security policies from-zone $UNZONES to-zone $INZONE policy $SET then deny" >>/tmp/blockpol
		echo "set logical-systems $LS_NAME security policies from-zone $UNZONES to-zone $INZONE policy $SET then log session-init" >>/tmp/blockpol
		echo "set logical-systems $LS_NAME security policies from-zone $UNZONES to-zone $INZONE policy $SET then count" >>/tmp/blockpol

		if [ "$ISTRUL" ]; then
			echo "insert logical-systems $LS_NAME security policies from-zone $INZONE to-zone $UNZONES policy $SET before policy $ISTRUL" >>/tmp/blockpol
		fi

		if [ "$ISTRULTU" ]; then
			echo "insert logical-systems $LS_NAME security policies from-zone $UNZONES to-zone $INZONE policy $SET before policy $ISTRULTU" >>/tmp/blockpol
		fi
	done

	return 0
}

############################################
FNSetPolicies() {

	echoMsg INFO "Setting policies"

	for INZONE in $INZONES; do
	
		ISTRUL=$(echo "show security policies from-zone $INZONE to-zone $UNZONES" | cli | awk '/Policy/ {print($2)}' | sed 's/,//;1q')	
		ISTRULTU=$(echo "show security policies from-zone $UNZONES to-zone $INZONE" | cli | awk '/Policy/ {print($2)}' | sed 's/,//;1q')

		FNSetAllowSet
		[ $? -ne 0 ] && return 1

		FNSetBlockSet		
		[ $? -ne 0 ] && return 1

	done

	return 0
}

############################################
FNSetPoliciesWithLS() {

	echoMsg INFO "Setting policies for [$LS_NAME] logical system"

	for INZONE in $INZONES; do
	
		ISTRUL=$(echo "show logical-systems $LS_NAME security policies from-zone $INZONE to-zone $UNZONES" | cli | awk '/Policy/ {print($2)}' | sed 's/,//;1q')	
		ISTRULTU=$(echo "show logical-systems $LS_NAME security policies from-zone $UNZONES to-zone $INZONE" | cli | awk '/Policy/ {print($2)}' | sed 's/,//;1q')

		FNSetAllowSetWithLS
		[ $? -ne 0 ] && return 1

		FNSetBlockSetWithLS		
		[ $? -ne 0 ] && return 1

	done

	return 0
}

############################################
FNCommitChanges() {

	echoMsg INFO "Commiting policy configuration"

	############################################
	echo "commit check" >> /tmp/blockpol
	echo "commit" >> /tmp/blockpol
	echo "exit" >> /tmp/blockpol

	############################################
	echo "commit check" >> /tmp/alopol
	echo "commit" >> /tmp/alopol
	echo "exit" >> /tmp/alopol

	############################################
	#cat /tmp/alopol
	#cat /tmp/blockpol

	echo
	echo "--------------------------------------------------------------------------------"
	echo "The policies to be create list is stored in /tmp/alopol and /tmp/blockpol file"
	echo "acordingly. Please verify the lists and when you are ready type yes to continue"
	echo "with policies deployment. If not sure type no and programm will be terminated"
	echo "--------------------------------------------------------------------------------"
	echo

	echo "Continue to deploy yes/no?"
	read -p "[yes]" opt

	opt=${opt:-'yes'}

	if [ "$opt" != "yes" ]; then
		echo -e "\tpolicies NOT committed....."
		echo -e "\tIf you want to check the commands in more detail or send them to ThreatSTOP to"
		echo -e "\treport a bug, they are /tmp/alopol and /tmp/blockpol"

		echo -e "\tif you want to apply these file manually, run the following commands:"
		echo -e "\t\tcat /tmp/alopol | /usr/sbin/cli"
		echo -e "\t\tcat /tmp/blockpol | /usr/sbin/cli"
		
		return 2
	fi

	echo
	echo
	echo -e "\t\t\t\033[41mProcessing ... May take some time... Please be patient...\033[0m"
	echo
	echo
	cat /tmp/alopol | /usr/sbin/cli  >$LOGFILE
	cat /tmp/blockpol | /usr/sbin/cli >>$LOGFILE

	return 0
}


##############################################################
###################### MAIN PART #############################
##############################################################
 
clear

echo
echo "****************************************************************"
echo "* Starting ThreatSTOP policies create operation                *"
echo "****************************************************************"
echo

setLock
[ $? -ne 0 ] && exiting

FNloadConfig
[ $? -ne 0 ] && exiting

FNgetZones
[ $? -ne 0 ] && exiting

FNConfigureSets
[ $? -ne 0 ] && exiting

if [ -z "$LS_NAME" ]; then
	FNSetPolicies
	[ $? -ne 0 ] && exiting
else 
	FNSetPoliciesWithLS
	[ $? -ne 0 ] && exiting
fi

FNCommitChanges
rc=$?
[ $rc -eq 1 ] && exiting
[ $rc -eq 2 ] && cleanup

unLock
[ $? -ne 0 ] && exiting

echo
echo "****************************************************************"
echo "* ThreatSTOP policies have been created on your device         *"
echo "****************************************************************"
echo

