#!/bin/sh

###############################################################################
# $Id: tsuninstall.sh 577 2013-01-21 15:05:20Z alein $
#
# Copyright(c) 2006-2011 ThreatSTOP, Inc. All Rights Reserved
#
# NOTICE:  All information contained herein is, and remains the property
# of ThreatSTOP, Inc. and its suppliers, if any. The intellectual and
# technical concepts contained herein are proprietary to ThreatSTOP, Inc.
# and its suppliers and may be covered by U.S. and Foreign Patents, patents
# in process, and are protected by trade secret or copyright law.
# Dissemination of this information or reproduction of this material is
# strictly forbidden unless prior written permission is obtained from
# ThreatSTOP, Inc.
#
#
# Uninstall ThreatSTOP from a JunOS Router. Removes the address book entries
# from the trust/untrust groups. Remves cron job that calls the script to
# download and apply the block lists.
###############################################################################

####################################################################
#################### GLOBAL VARIABLES ##############################
####################################################################

VERSION="0.98"
GETBLCKLIST_SCRIPT="tsgetblocklist.sh"
LOCKFILE="lock.lck"
CONFIG="threatstop.conf"
MYUID=$(id -u)
XSLT_PATH="/var/db/scripts/commit"

DEBUG="false"
[ "$1" = "DEBUG" ] && DEBUG="true"

#Changing to working dir
DIR=$(dirname $0)
[ "$DIR" != '.' ] && cd $DIR

####################################################################
###################### FUNCTIONS PART ##############################
####################################################################

############################################
echoMsg() {

        case $1 in
                ERROR) echo "[ERROR] : $2"
                        ;;
                WARN)  echo "[WARN ] : $2"
                        ;;
                ALERT) echo "[ALERT] : $2"
                        ;;
                INFO)  echo "[INFO ] : $2"
                        ;;
                DEBUG) 
			if [ $DEBUG = true ]; then
                                echo -e "\t[DEBUG] : $2"
                       	fi
                        ;;
                *) echo "SCRIPT ERROR : Unrecognized parameter [$1]"
        esac
}

############################################
exitting() {

        echo
        echo "****************************************************************"
        echo "*Unrecoverable errors enconted ... aborting...                 *"
        echo "*Please fix then run script again                              *"
        echo "****************************************************************"
        echo
        exit 1
}

############################################
setLock() {

        trap "/bin/rm -f $LOCKFILE; return 1" 0 1 2 3 15

        echoMsg INFO "Locking current instance"

        if [ -e $LOCKFILE ]; then
                ps=$(cat $LOCKFILE)
                echoMsg ERROR "Failed to lock ... Another instance [$ps] of the same script is runnng"
                return 1
        fi

        echo $$ > $LOCKFILE
        return 0
}

############################################
unLock() {

        echoMsg INFO "Unlocking current instance"
        /bin/rm -f $LOCKFILE
        return 0
}

############################################
verifyUserID() {

	echoMsg INFO "Verifying our credential"

        if [ $MYUID -ne 0 ]; then
                echoMsg ERROR "Wrong UID ... This script must be run under root credentials ..."
                return 1
        fi

        return 0
}

###########################################
FNloadConfig() {

        echoMsg INFO "Loading configurations"

        if [ ! -f $CONFIG ]; then
                        echoMsg ERROR "Cannot find config file: \"$CONFIG\""
                        return 1
        fi
        . $CONFIG

        return 0
}

##############################################
FNgetRules() {

	echoMsg INFO "Getting security policies list"

	tsruls=$(echo "
		configure
		show security policies
	"| /usr/sbin/cli)

	thtreav=$(echo $tsruls | sed -n -e '/ThreatSTOP/p'| wc -l)

	if [ $thtreav -gt 0 ]; then
        	echoMsg ALERT "The policies using ThreatSTOP address-sets were found"
		echoMsg ALERT "You should remove these policies in order to proceed with uninstall."
        	return 1
	fi

	return 0
}

##############################################
FNRemoveThreatSTOPEntriesDo() {

	local fname=$1

	echoMsg INFO "Removing ThreatSTOP entries from device configuration"
	echo "configure
		set system scripts commit file $fname
		delete system syslog file $DEVICE_IP.log
		commit check
		commit
	" | /usr/sbin/cli


	return 0
}

##############################################
FNRemoveThreatSTOPEntries() {

	echoMsg INFO "Ready to remove ThreatSTOP enties from the system"

	if [ "$ADDRESS_BOOK_TYPE" = "Zones" ]; then
		echoMsg INFO "Removing Zones address books"
		FNRemoveThreatSTOPEntriesDo "tsuninstallZones.xsl"
		return 0
	fi

	if [ "$ADDRESS_BOOK_TYPE" = "Security" ]; then
		echoMsg INFO "Prepare unistall commit script"
		if [ ! -e "${XSLT_PATH}/tsuninstallSecurity.xsl" ]; then
			cat scripts/tsuninstallSecurity.xsl > /var/db/scripts/commit/tsuninstallSecurity.xsl
		fi

		echoMsg INFO "Removing Security address books"
		FNRemoveThreatSTOPEntriesDo "tsuninstallSecurity.xsl"
                return 0
        fi

	return 0
}

##############################################
FNCleanUPXLSTscripts() {

	echoMsg INFO "Removing ThreatSTOP commit files from the system"

	if [ -e "${XSLT_PATH}/tsuninstallSecurity.xsl" ]; then
		echoMsg DEBUG "Removing - tsuninstallSecurity.xsl"
		rm -f ${XSLT_PATH}/tsuninstallSecurity.xsl
	fi

	if [ -e "${XSLT_PATH}/tsuninstallZones.xsl" ]; then
		echoMsg DEBUG "Removing - tsuninstallZones.xsl"
		rm -f ${XSLT_PATH}/tsuninstallZones.xsl
	fi

	if [ -e "${XSLT_PATH}/tsinstallZones.xsl" ]; then
		echoMsg DEBUG "Removing - tsinstallZones.xsl"
		rm -f ${XSLT_PATH}/tsinstallZones.xsl
	fi

	if [ -e "${XSLT_PATH}/tsinstallSecurity.xsl" ]; then
		echoMsg DEBUG "Removing - tsinstalliSecurity.xsl"
		rm -f ${XSLT_PATH}/tsinstallSecurity.xsl
	fi

	return 0
}
##############################################
FNCleanCronTab() {

	echoMsg INFO "Cleaning crontab entries"

	#echo "Removing the cron job that updates the address book"
	cronUser=` /usr/sbin/crontab -l | sed -e "/$GETBLCKLIST_SCRIPT/d"`
	echo "yes" | /usr/sbin/crontab -r
	echo "$cronUser" |  /usr/sbin/crontab -

	return 0
}

####################################################################
########################## MAIN PART ###############################
####################################################################

clear           
        
echo
echo "****************************************************************"
echo "* Starting ThreatSTOP uninstall operation                      *"
echo "****************************************************************"
echo
                        
setLock
[ $? -ne 0 ] && exitting
                        
verifyUserID
[ $? -ne 0 ] && exitting
        
FNloadConfig
[ $? -ne 0 ] && exitting

FNgetRules
[ $? -ne 0 ] && exitting

FNRemoveThreatSTOPEntries
[ $? -ne 0 ] && exitting

#FNCleanUPXLSTscripts
#[ $? -ne 0 ] && exitting

FNCleanCronTab
[ $? -ne 0 ] && exitting

unLock
[ $? -ne 0 ] && exitting

echo
echo "*******************************************************************"
echo "* ThreatSTOP address books have been uninstalled from your device *"
echo "*******************************************************************"
echo

